class MMPlayerData expands ReplicationInfo;

struct PreciseVector
{
	var() config float X;
	var() config float Y;
	var() config float Z;
};

var Pawn POwner;
var float BaseEyeHeight;
var int OwnerID;
var MMPRI OwnerPRI;
var int RealHealth;
var vector LastRepLoc;
var float LastRepLocTime;
var bool bReplicateLoc;
var bool bReplicateHealth;
var bool bHudEnforceHealth;


replication
{
	unreliable if ( !bNetOwner && Role==ROLE_Authority )
		BaseEyeHeight;
	unreliable if ( !bNetOwner && bReplicateHealth && Role==ROLE_Authority )
		RealHealth;
	reliable if ( Role==ROLE_Authority )
		OwnerID;

	unreliable if ( Role==ROLE_Authority )
		AdjustPlayerLocation;
}

event PostBeginPlay()
{
	POwner = Pawn(Owner);
	if ( Owner.IsA('bbPlayer') )
		bReplicateHealth = true;
	OwnerPRI = MMPRI(POwner.PlayerReplicationInfo);
	bReplicateLoc = true;
}

simulated event SetInitialState()
{
	bScriptInitialized = true;
	if ( Level.NetMode != NM_Client )
		GotoState('Server','InitID');
}

simulated event PostNetBeginPlay()
{
	GotoState('Client');
}


state Server
{
InitID:
	Sleep(0.0);
	OwnerID = POwner.PlayerReplicationInfo.PlayerID;
	RemoteRole = ROLE_SimulatedProxy;
Begin:
	if ( POwner == none || POwner.bDeleteMe )
	{
		Destroy();
		Stop;
	}
	BaseEyeHeight = POwner.BaseEyeHeight;
	if ( POwner.bHidden )	NetUpdateFrequency = 1.5;
	else					NetUpdateFrequency = POwner.NetUpdateFrequency * 0.5;
	if ( ShouldReplicateLoc() )
		ReplicateLoc();
	if ( bReplicateHealth )
		RealHealth = POwner.Health;
	Sleep(0.0);
	Goto('Begin');
}

simulated state Client
{
Begin:
	if ( !FindPRI() )
	{
		Sleep(0.3);
		Goto('Begin');
	}
FindPawn:
	POwner = Pawn(OwnerPRI.Owner);
	if ( POwner != none )
		SetOwner( POwner);
	if ( (PlayerPawn(POwner) != none) && ViewPort(PlayerPawn(POwner).Player) != none )
		Goto('OwnerClient');
SetProps:
	if ( POwner == none || POwner.bDeleteMe )
	{
		Sleep(0.0);
		Goto('FindPawn');
	}
	POwner.BaseEyeHeight = BaseEyeHeight;
	POwner.EyeHeight = BaseEyeHeight;
	Sleep(0.0);
	Goto('SetProps');
OwnerClient:
	Sleep(5);
	if ( POwner.IsA('bbPlayer') && MMHUD(PlayerPawn(POwner).myHud) != none )
		MMHUD(PlayerPawn(POwner).myHud).bEnforceHealth = true;
	Stop;
}

simulated function SetHealth()
{
	if ( (POwner != none) && (RealHealth != -1337) )
		POwner.Health = RealHealth;
}

simulated function bool FindPRI()
{
	local MMPRI PRI;

	ForEach AllActors (class'MMPRI', PRI)
		if ( PRI.PlayerID == OwnerID )
		{
			if ( PRI.bAdmin && PRI.bIsSpectator )
				continue;
			OwnerPRI = PRI;
			return true;
		}
}

function bool ShouldReplicateLoc()
{
		if ( (LastRepLocTime < (Level.TimeSeconds - 1 * Level.TimeDilation)) || (POwner.Location != LastRepLoc) )
			return POwner.FastTrace( POwner.Location - vect(0,0,1.2) * POwner.CollisionHeight);
}

function ReplicateLoc()
{
	local MMPlayerData MMP;
	LastRepLoc = POwner.Location;
	LastRepLocTime = Level.TimeSeconds;
	ForEach AllActors (class'MMPlayerData', MMP)
		if ( MMP != self )
			MMP.AdjustPlayerLocation( POwner, POwner.Location.X, POwner.Location.Y, POwner.Location.Z);
}

simulated function AdjustPlayerLocation( Pawn Other, float LX, float LY, float LZ)
{
	local vector aVec;
	local bool bOldCol;
	aVec.X = LX; aVec.Y = LY; aVec.Z = LZ;
	bOldCol = Other.bCollideWorld;
	Other.bCollideWorld = false;
	Other.bCollideWhenPlacing = false;
	Other.SetLocation( aVec);
	Other.bCollideWorld = bOldCol;
	Other.bCollideWhenPlacing = true;
}

static final function vector ToVector( PreciseVector A)
{
	local vector V;
	V.X = A.X;
	V.Y = A.Y;
	V.Z = A.Z;
	return V;
}
static final function bool IsZero( PreciseVector A)
{
	return (A.X == 0) && (A.Y == 0) && (A.Z == 0);
}

defaultproperties
{
     OwnerID=-1
     RealHealth=-1337
}
