//=============================================================================
// MMScore
//=============================================================================
class MMScore extends TeamScoreBoard;

#exec TEXTURE IMPORT FILE=Textures\ScoreBoard\Shade.pcx GROUP="ScoreBoard"
#exec TEXTURE IMPORT FILE=Textures\ScoreBoard\Shade2.pcx GROUP="ScoreBoard"
#exec TEXTURE IMPORT FILE=Textures\ScoreBoard\Monster.pcx GROUP="ScoreBoard"

struct FlagData {
	var string Prefix;
	var texture Tex;
};

var FlagData FD[32];
var Pawn PawnList[32];
var int counter, tableWidth, tableHeaderHeight, cellHeight, countTeams, saveindex, TeamPlayers[4], 
		ShowMaxPlayer1, ShowMaxPlayer2, NotShownPlayers[4], LastSortTime, countPawns, tableLine1,
		tableLine2, paddingInfo, avgEff[4], avgPi[4], avgPl[4], avgY;
var() Color White, Pink, TealColor, Orange, PurpleColor,
		  RedColor, BlueColor, GreenColor, GoldColor, getHeaderColor[4], getTeamColor[4];
var Font PtsFont30,PtsFont26,PtsFont24,PtsFont22, PtsFont20, PtsFont18, PtsFont16, PtsFont14, PtsFont12;
var Texture getIconTexture[4], getTeamIcon[4], getHeaderTexture[4];
var string teamNames[4];
var float TeamX[4], TeamY[4];
var MMGameReplicationInfo MMGRI;
var TournamentGameReplicationInfo OwnerGame;

function PostBeginPlay()
{
  super.PostBeginPlay();
  LastSortTime = -100;
  
  MMGRI = MMGameReplicationInfo( PlayerPawn(Owner).GameReplicationInfo);
  
  PtsFont30 = Font( DynamicLoadObject( "LadderFonts.UTLadder30", class'Font' ) );
  PtsFont26 = Font( DynamicLoadObject( "LadderFonts.UTLadder22", class'Font' ) );
  PtsFont24 = Font( DynamicLoadObject( "LadderFonts.UTLadder22", class'Font' ) );
  PtsFont22 = Font( DynamicLoadObject( "LadderFonts.UTLadder22", class'Font' ) );
  PtsFont20 = Font( DynamicLoadObject( "LadderFonts.UTLadder20", class'Font' ) );
  PtsFont18 = Font( DynamicLoadObject( "LadderFonts.UTLadder18", class'Font' ) );
  PtsFont16 = Font( DynamicLoadObject( "LadderFonts.UTLadder16", class'Font' ) );
  PtsFont14 = Font( DynamicLoadObject( "LadderFonts.UTLadder14", class'Font' ) );
  PtsFont12 = Font( DynamicLoadObject( "LadderFonts.UTLadder12", class'Font' ) );
}

function ShowScores(Canvas Canvas){
	local int i, Time, eff;
	local float X,Y, xLen,yLen, pnx,pny, paddingInfo;
	local string s;
	local Pawn p;
	local int TeamExist[4];

	OwnerGame = TournamentGameReplicationInfo(PlayerPawn(Owner).GameReplicationInfo);
	
	if(Canvas.ClipX < 900){
		tableWidth = 370;
		paddingInfo = 160;
	}
	else{
		tableWidth = Self.default.tableWidth;
		paddingInfo = Self.default.paddingInfo;
	}

	if(Canvas.ClipY<750){
		tableLine2 = 300;
		ShowMaxPlayer1=9;
		ShowMaxPlayer2=3;
	}
	else{
		tableLine2 = Self.default.tableLine2;
		ShowMaxPlayer1 = Self.default.ShowMaxPlayer1;
		ShowMaxPlayer2 = Self.default.ShowMaxPlayer2;
	}
	
	if( Level.TimeSeconds - LastSortTime > 0.5 )
	{
	  sortPawn();
	  LastSortTime = Level.TimeSeconds;
	}


	for(i=0; i<4; i++){
		if( (MMGRI.Queens[i]!=None) && !MMGRI.Queens[i].bDeleteMe ){
			TeamExist[i] = 1;
		}
	}

	for ( i = 0; i < countPawns; i++ ){
		TeamExist[PawnList[i].PlayerReplicationInfo.Team] = 1;
	}
	
	countTeams = 0;
	for(i = 0; i < 4; i++ )
	{
		TeamPlayers[i] = 0;
		NotShownPlayers[i] = 0;

		if(TeamExist[i]>0)
		{
			X = getXHeader(Canvas.ClipX);
			Y = getYHeader();
			TeamX[i] = X;
			TeamY[i] = Y; 
			
			////////
			//Header
			////////
			Canvas.bNoSmooth = False;
			//Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.DrawColor = getHeaderColor[i];
			Canvas.Style = ERenderStyle.STY_Modulated;
			Canvas.SetPos( X, Y );
			Canvas.DrawRect( texture'shade2', tableWidth , tableHeaderHeight );

			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.SetPos( X, Y );
		if ( getHeaderTexture[i] != none )
			Canvas.DrawPattern( getHeaderTexture[i], tableWidth , tableHeaderHeight , 1 );


			//Header icons
			
			Canvas.DrawColor = getTeamColor[i];

			Canvas.SetPos( X+5, Y+5 );
			Canvas.DrawIcon(getTeamIcon[i], 0.5 );
			Canvas.Font = PtsFont26;
			Canvas.Style = ERenderStyle.STY_Normal;
			Canvas.SetPos( X+50, Y + 10);
			Canvas.DrawText(teamNames[i]);
			Canvas.StrLen(teamNames[i],xLen,yLen);

			//avgInfo
			Canvas.Font = Font'SmallFont';
			Canvas.Style = ERenderStyle.STY_Normal;
			Canvas.StrLen("AVG",pnx,pny);
			avgY = (tableHeaderHeight - (3*pny))/4;
			Canvas.SetPos(X+70+xLen, Y + avgY);
			Canvas.DrawText("PI:"@avgPi[i]@"ms");
			Canvas.SetPos(X+70+xLen, Y + 2*avgY + pny);
			Canvas.DrawText("PL:"@avgPl[i]$"%");
			Canvas.SetPos(X+70+xLen, Y + 3*avgY + 2*pny );
			Canvas.DrawText("EFF:"@avgEff[i]$"%");

			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.Font = Font'LEDFont2';
			Canvas.StrLen(int(OwnerGame.Teams[i].Score),xLen,yLen);
			
			Canvas.SetPos(X+tableWidth-xLen-42, Y+5 );
			Canvas.DrawIcon(getIconTexture[i], 0.5 );
			
			Canvas.Style = ERenderStyle.STY_Normal;
			Canvas.SetPos( X+tableWidth-xLen-5, Y + 5);
			Canvas.DrawText(int(OwnerGame.Teams[i].Score), false);
			
			avgEff[i] = 0;
			avgPi[i] = 0;
			avgPl[i] = 0;
			
			countTeams++;
		}
	}
	
	for ( i = 0; i < countPawns; i++)
	{
		p = PawnList[i];
		if(p.PlayerReplicationInfo != None && (!p.PlayerReplicationInfo.bIsSpectator || p.PlayerReplicationInfo.bWaitingPlayer) && !p.IsA('Spectator'))
		{
			if(countTeams>2 && TeamPlayers[p.PlayerReplicationInfo.Team] >= ShowMaxPlayer2)
				NotShownPlayers[p.PlayerReplicationInfo.Team]++;
			else if(countTeams<=2 && TeamPlayers[p.PlayerReplicationInfo.Team] >= ShowMaxPlayer1)
				NotShownPlayers[p.PlayerReplicationInfo.Team]++;
			else
			{
				X = TeamX[p.PlayerReplicationInfo.Team]; 
				Y = TeamY[p.PlayerReplicationInfo.Team] + tableHeaderHeight + TeamPlayers[p.PlayerReplicationInfo.Team] * cellHeight;

				Canvas.Style = ERenderStyle.STY_Modulated;
				Canvas.SetPos( X,Y);  
				Canvas.DrawRect( texture'shade2', tableWidth , cellHeight );		 
	  
				//face
			  Canvas.DrawColor = White;
			  Canvas.Style = ERenderStyle.STY_Translucent;
			  Canvas.SetPos( X+5,Y+5);

			  if( p.PlayerReplicationInfo.TalkTexture != None ) Canvas.DrawIcon( p.PlayerReplicationInfo.TalkTexture, 0.5 * 64.0 / p.PlayerReplicationInfo.TalkTexture.VSize );

			  else if(p.IsA('ScriptedPawn'))
				  Canvas.DrawIcon( texture'Monster', 0.5 );

			  else Canvas.DrawIcon( texture'TLSteel', 0.5 );	  
	  
			  //name
			  Canvas.Font = PtsFont20;

			  if(p.PlayerReplicationInfo.bAdmin){
				  Canvas.DrawColor = White;
			  }

			  else if(p.IsA('ScriptedPawn')){
				  Canvas.DrawColor = getTeamColor[p.PlayerReplicationInfo.Team];
			  }

			  else if(p.PlayerReplicationInfo.PlayerID == PlayerPawn(Owner).PlayerReplicationInfo.PlayerID){
				  Canvas.DrawColor = Pink;
			  }

			  else{
				  Canvas.DrawColor = getTeamColor[p.PlayerReplicationInfo.Team];
			  }

			  Canvas.Style = ERenderStyle.STY_Normal;
			  Canvas.SetPos( X + 45, Y + 7);
			  Canvas.StrLen(p.PlayerReplicationInfo.PlayerName,pnx,pny);
			  Canvas.DrawText(p.PlayerReplicationInfo.PlayerName);
			  
			  //ping-packetloss-bot
			  Canvas.Font = Font'SmallFont';
			  Canvas.SetPos( X + 45, Y + pny + 7);

			  if(p.IsA('Bot'))
				  Canvas.DrawText("BOT");

			  else if(p.IsA('ScriptedPawn'))
				  Canvas.DrawText("Monster");

			  else{
				  Canvas.StrLen("PI:"@p.PlayerReplicationInfo.Ping@"ms | PL:"@p.PlayerReplicationInfo.PacketLoss$"%",xLen,yLen);
				  Canvas.DrawText("PI:"@p.PlayerReplicationInfo.Ping@"ms | PL:"@p.PlayerReplicationInfo.PacketLoss$"%");
			  }
			  
			  //country flag
			  if(!p.IsA('Bot') && MMPRI(p.PlayerReplicationInfo).CountryPrefix != "")
			  {
				Canvas.SetPos(X+xLen+90, Y + pny + 7);
				Canvas.DrawColor = WhiteColor;
				Canvas.DrawIcon(FD[GetFlagIndex(MMPRI(p.PlayerReplicationInfo).CountryPrefix)].Tex, 1.0);
			  }
		  
			  // Draw Monsters
	  		  Canvas.DrawColor=getTeamColor[3];
			  Canvas.SetPos(X+ paddingInfo+25, Y + 7);
			  Canvas.StrLen("Ping:     ", xLen, yLen);
			  Canvas.DrawText("");
		  
			  // Draw Time
			  Canvas.DrawColor=White;
			  Time = Max(1, (Level.TimeSeconds + PlayerPawn(Owner).PlayerReplicationInfo.StartTime - p.PlayerReplicationInfo.StartTime)/60);
			  Canvas.SetPos(X+xLen+paddingInfo+45, Y + 7);
			  Canvas.DrawText(TimeString$":"$Time, false);
		  
			  // Draw Health
			  if(p.PlayerReplicationInfo.Team == Pawn(Owner).PlayerReplicationInfo.Team)
			  {
			  Canvas.DrawColor = getTeamColor[2];
			  Canvas.SetPos(X+ paddingInfo+25, Y + 7);
			  Canvas.StrLen("Ping:     ", xLen, yLen);
			  Canvas.DrawText("Health:"$(p.Health), false);
			  }

	  		  // Draw Credit
			  if(p.PlayerReplicationInfo.Team == Pawn(Owner).PlayerReplicationInfo.Team)
			  {
			  Canvas.DrawColor = PurpleColor;
			  Canvas.SetPos(X+paddingInfo+25, Y + yLen + 11);
			  Canvas.StrLen("Ping:     ", xLen, yLen);
			  Canvas.DrawText("Credit:"$(p.Weapon.AmmoType.AmmoAmount), false);
			  }
		  
			  // Draw Deaths
			  Canvas.DrawColor=RedColor;
			  Canvas.SetPos(X+xLen+paddingInfo+45, Y + yLen + 11);
			  Canvas.DrawText("Dths:"@int(p.PlayerReplicationInfo.Deaths), false);
	  	
			  // Draw Scores
			  Canvas.Font = PtsFont20;
			  Canvas.DrawColor = getTeamColor[p.PlayerReplicationInfo.Team];
			  Canvas.StrLen(""@int(p.PlayerReplicationInfo.Score),xLen,yLen);
			  Canvas.SetPos(X+tableWidth-xLen-5, Y + 7);
			  Canvas.DrawText(""@int(p.PlayerReplicationInfo.Score), false);
			  
			  // Draw Monster & Costs
			  if(p.PlayerReplicationInfo.Team == Pawn(Owner).PlayerReplicationInfo.Team)
			  {
				Canvas.Font = Font'SmallFont';
				Canvas.DrawColor = TealColor;
				Canvas.StrLen(""$(p.Weapon.ItemName$p.Weapon.M_Activated),xLen,yLen);
				Canvas.SetPos(X+tableWidth-xLen-5, Y + pny + 7);
				Canvas.DrawText(""$(p.Weapon.ItemName$p.Weapon.M_Activated), false);

			  }
			  
			}

			if ( p.PlayerReplicationInfo.Team < 4 )
			{
				TeamPlayers[p.PlayerReplicationInfo.Team]++;
				avgEff[p.PlayerReplicationInfo.Team] += eff;
				avgPi[p.PlayerReplicationInfo.Team] += p.PlayerReplicationInfo.Ping;
				avgPl[p.PlayerReplicationInfo.Team] += p.PlayerReplicationInfo.PacketLoss;
			}
		}
	}

	for(i=0;i<4;i++)
	{
		avgEff[i] = avgEff[i] / TeamPlayers[i];
		avgPi[i] = avgPi[i] / TeamPlayers[i];
		avgPl[i] = avgPl[i] / TeamPlayers[i];
		
		if(NotShownPlayers[i]>0)
		{
			if(countTeams <= 2)
			{
				X = TeamX[i];
				Y = TeamY[i] + tableHeaderHeight + ShowMaxPlayer1 * cellHeight;				
			}
			else
			{
				X = TeamX[i];
				Y = TeamY[i] + tableHeaderHeight + ShowMaxPlayer2 * cellHeight; 
			}
			
			Canvas.Style = ERenderStyle.STY_Modulated;
			Canvas.SetPos( X,Y);
			Canvas.DrawRect( texture'shade2', tableWidth , cellHeight-10 );
			
			Canvas.DrawColor = getTeamColor[i];
			Canvas.Style = ERenderStyle.STY_Normal;
			Canvas.Font = PtsFont16;
			Canvas.SetPos( X+5,Y+5);
			Canvas.DrawText(NotShownPlayers[i]@"Player not shown!", false);
		}
	}
	DrawFooters(Canvas);
}

function DrawFooters( Canvas C )
{
  local float DummyX, DummyY, Nil, X1, Y1;
  local string TextStr;
  local string TimeStr, HeaderText;
  local int Hours, Minutes, Seconds, i;
  local PlayerReplicationInfo PRI;
  local color specColor;
  local int baseX, baseY;
  local TournamentGameReplicationInfo TGRI;
  local float XL, YL;

  C.bCenter = True;
  C.Font = MyFonts.GetSmallestFont( C.ClipX );

  // Display server info in bottom center
  C.DrawColor = getTeamColor[2];
  C.StrLen( "Test", DummyX, DummyY );
  C.SetPos( 0, C.ClipY - DummyY );
  TextStr = "Playing" @ (PlayerPawn(Owner).GameReplicationInfo.GameName) @ "in"  @ Level.Title @ "on" @ (PlayerPawn(Owner).GameReplicationInfo.ServerName);
  C.DrawText( TextStr );

  C.DrawColor = Pink;
  C.Style = ERenderStyle.STY_Normal;
  C.SetPos( 0, C.ClipY - 3 * DummyY );
  C.DrawText("Map Author: "$Level.Author, true);

	TGRI = TournamentGameReplicationInfo(PlayerPawn(Owner).GameReplicationInfo);
	if ( TGRI == None )
		return;

  // Draw Time
  if( ( PlayerPawn(Owner).GameReplicationInfo.RemainingTime > 0 ) )
  {

	if( PlayerPawn(Owner).GameReplicationInfo.RemainingTime <= 0 )
	{
	  TimeStr = RemainingTime $ "00:00";
	}
	else
	{
	  Minutes = PlayerPawn(Owner).GameReplicationInfo.RemainingTime / 60;
	  Seconds = PlayerPawn(Owner).GameReplicationInfo.RemainingTime % 60;
	  TimeStr = RemainingTime $ TwoDigitString( Minutes ) $ ":" $ TwoDigitString( Seconds );
	}
  }
  else
  {
	Seconds = PlayerPawn(Owner).GameReplicationInfo.ElapsedTime;
	Minutes = Seconds / 60;
	Hours = Minutes / 60;
	Seconds = Seconds - ( Minutes * 60 );
	Minutes = Minutes - ( Hours * 60 );
	TimeStr = ElapsedTime $ TwoDigitString( Hours ) $ ":" $ TwoDigitString( Minutes ) $ ":" $ TwoDigitString( Seconds );
  }
	ForEach AllActors (class'PlayerReplicationInfo', PRI)
	{
		if ( PRI.bIsSpectator && !PRI.bWaitingPlayer && PRI.StartTime > 0)
		{
			if(HeaderText=="") HeaderText = pri.Playername;
			else HeaderText = HeaderText$", "$pri.Playername;
		}
	}
	if (HeaderText=="") HeaderText = "No one spectating this match."; else HeaderText = HeaderText$"."; // I'm sorry about this, it's really stupid																													 //  but I'm to lazy rewrite it :P
																														 // Atleast it's working..
  C.SetPos( 0, C.ClipY - 2 * DummyY );
  C.DrawColor = Orange;
  C.DrawText( "Current Time:" @ GetTimeStr() @ "|" @ FragGoal@TGRI.GoalTeamScore @ "|" @ "Time Limit:" @TGRI.TimeLimit$":00" @ "|" @ TimeStr );

  // Draw Spectators
  C.StrLen( HeaderText, DummyX, Nil );
  C.Style = ERenderStyle.STY_Normal;
  C.SetPos( 0, C.ClipY - 4.2 * DummyY );
  
  C.Font = MyFonts.GetSmallestFont( C.ClipX );
  C.DrawColor = getTeamColor[3];
  C.DrawText("Spectators:"@HeaderText);

  HeaderText="";

  C.bCenter = False;

}

function sortPawn(){
	local int i,j, maxIndex;
	local Pawn p, maxValue;

	countPawns=0;

	foreach AllActors(class'Pawn', p){
		if(p.PlayerReplicationInfo != None && !p.IsA('Spectator')){
			PawnList[countPawns] = p;
			countPawns++;
			if(countPawns>=32) break;
		}
	}

	for(i=0; i<countPawns; i++){
		maxIndex = i;
		maxValue = PawnList[i];
		for(j=i+1; j<countPawns; j++){
			if(PawnList[j].PlayerReplicationInfo.Score > maxValue.PlayerReplicationInfo.Score){
				maxValue = PawnList[j];
				maxIndex = j;
			}
		}
		PawnList[maxIndex] = PawnList[i];
		PawnList[i] = maxValue;
	}
	
}

function float getXHeader(int screenWidth){
	local float x;
	x = (screenWidth-2*tableWidth)/3; 
	switch(countTeams){
		case 0: 
		case 2: return x;
		case 1: 
		case 3: return 2*x+tableWidth;
		default: return 0;
	}
}

function float getYHeader(){
	
	switch(countTeams){
		case 0: 
		case 1: return tableLine1;
		case 2: 
		case 3: return tableLine2;
		default: return 0;
	}
}




function int GetFlagIndex(string Prefix)
{
	local int i;
	for(i=0;i<32;i++)
		if(FD[i].Prefix == Prefix)
			return i;
	FD[saveindex].Prefix=Prefix;
	FD[saveindex].Tex=texture(DynamicLoadObject("CountryFlags2."$Prefix, class'Texture'));
	i=saveindex;
	saveindex = (saveindex+1) % 256;
	return i;
}

function string GetTimeStr()
{
  local string Mon, Day, Hour, Min;

  Hour = string( PlayerPawn( Owner ).Level.Hour );
  if( int( Hour ) < 10 ) Hour = "0" $ Hour;
  
  Min = string( PlayerPawn( Owner ).Level.Minute );
  if( int( Min ) < 10 ) Min = "0" $ Min;

  switch( PlayerPawn( Owner ).Level.month )
  {
	case  1: Mon = "Jan"; break;
	case  2: Mon = "Feb"; break;
	case  3: Mon = "Mar"; break;
	case  4: Mon = "Apr"; break;
	case  5: Mon = "May"; break;
	case  6: Mon = "Jun"; break;
	case  7: Mon = "Jul"; break;
	case  8: Mon = "Aug"; break;
	case  9: Mon = "Sep"; break;
	case 10: Mon = "Oct"; break;
	case 11: Mon = "Nov"; break;
	case 12: Mon = "Dec"; break;
  }

  switch( PlayerPawn( Owner ).Level.dayOfWeek )
  {
	case 0: Day = "Sunday";	break;
	case 1: Day = "Monday";	break;
	case 2: Day = "Tuesday";   break;
	case 3: Day = "Wednesday"; break;
	case 4: Day = "Thursday";  break;
	case 5: Day = "Friday";	break;
	case 6: Day = "Saturday";  break;
  }

  return Day @ PlayerPawn( Owner ).Level.Day @ Mon @ PlayerPawn( Owner ).Level.Year $ "," @ Hour $ ":" $Min;
}

defaultproperties
{
     tableWidth=450
     tableHeaderHeight=40
     cellHeight=40
     ShowMaxPlayer1=12
     ShowMaxPlayer2=5
     tableLine1=100
     tableLine2=410
     paddingInfo=200
     White=(R=255,G=255,B=255)
     Pink=(R=242,G=128,B=249)
     TealColor=(G=255,B=255)
     Orange=(R=255,G=128)
     PurpleColor=(R=180,G=64,B=255)
     RedColor=(R=255)
     BlueColor=(B=255)
     GreenColor=(G=255)
     GoldColor=(R=255,G=255)
     getHeaderColor(0)=(R=32)
     getHeaderColor(1)=(B=16)
     getHeaderColor(2)=(G=32)
     getHeaderColor(3)=(R=64,G=64)
     getTeamColor(0)=(R=255)
     getTeamColor(1)=(B=255)
     getTeamColor(2)=(G=255)
     getTeamColor(3)=(R=255,G=255)
     getTeamIcon(0)=Texture'Botpack.Icons.I_TeamR'
     getTeamIcon(1)=Texture'Botpack.Icons.I_TeamB'
     getTeamIcon(2)=Texture'Botpack.Icons.I_TeamG'
     getTeamIcon(3)=Texture'Botpack.Icons.I_TeamY'
     TeamNames(0)="Red Team"
     TeamNames(1)="Blue Team"
     TeamNames(2)="Green Team"
     TeamNames(3)="Gold Team"
     FragGoal="Team Score Limit:"
}
