class MMScoreBoard expands TeamScoreBoard;

var bool ShowPacketLoss;
var bool ShowNetSpeed;
var() Color White, Pink, TealColor, Orange, PurpleColor, RedColor, BlueColor, GreenColor, GoldColor;
var() string Playing;
var TournamentGameReplicationInfo TGRI;

struct FlagData {
	var string Prefix;
	var texture Tex;
};
var FlagData FD[32]; // there can be max 32 so max 32 different flags
var int saveindex; // new loaded flags will be saved in FD[index]

function int GetFlagIndex(string CountryFlagsPackage, string Prefix)
{
	local int i;
	for(i=0;i<32;i++)
		if(FD[i].Prefix == Prefix)
			return i;
	FD[saveindex].Prefix=Prefix;
	FD[saveindex].Tex=texture(DynamicLoadObject(CountryFlagsPackage$"."$Prefix, class'Texture'));
	i=saveindex;
	saveindex = (saveindex+1) % 256;
	return i;
}

function ShowScores( canvas Canvas )
{
	local PlayerReplicationInfo PRI;
	local int PlayerCount, i;
	local float XL, YL, XL2, YL2, XOffset, YOffset;
	local Font CanvasFont;
	local int PlayerCounts[4], TeamPing[4], TeamPacketLoss[4];

	Super.ShowScores(Canvas);

	CanvasFont = Canvas.Font;
	Canvas.DrawColor = WhiteColor;
	Canvas.Font = MyFonts.GetMediumFont( Canvas.ClipX );
	Canvas.StrLen("TEXT", XL, YL);
	Canvas.Font = Font'SmallFont';

	for ( i=0; i<32; i++ )
	{
		if (PlayerPawn(Owner).GameReplicationInfo.PRIArray[i] != None)
		{
			PRI = PlayerPawn(Owner).GameReplicationInfo.PRIArray[i];
			if ( !PRI.bIsSpectator || PRI.bWaitingPlayer )
			{
				PlayerCount++;
				PlayerCounts[PRI.Team]++;
				TeamPing[PRI.Team] += PRI.Ping;
				TeamPacketLoss[PRI.Team] += PRI.PacketLoss;
			}
		}
	}

	for ( i=0; i<4; i++ )
	{
		if ( PlayerCounts[i] > 0 )
		{
			if ( i % 2 == 0 )
				XOffset = (Canvas.ClipX / 4) - (Canvas.ClipX / 8);
			else
				XOffset = ((Canvas.ClipX / 4) * 3) - (Canvas.ClipX / 8);
			YOffset = ScoreStart - YL + 2;

			if ( i > 1 )
				if (PlayerCounts[i-2] > 0)
					YOffset = ScoreStart + YL*10;

			Canvas.StrLen(PingString$":     ", XL2, YL2);
			Canvas.SetPos(XOffset - XL2 - 6, YOffset + 2);
			Canvas.DrawText(PingString$":"@(TeamPing[i]/PlayerCounts[i]), false);

			// Draw Packetloss
			Canvas.StrLen(LossString$":    %", XL2, YL2);
			Canvas.SetPos(XOffset - XL2 - 6, YOffset + YL2 +3);
			Canvas.DrawText(LossString$":"@(TeamPacketLoss[i]/PlayerCounts[i])$"%", false);

		}
	}

	Canvas.Font = CanvasFont;
}

function DrawNameAndPing(Canvas Canvas, PlayerReplicationInfo PRI, float XOffset, float YOffset, bool bCompressed)
{
	local float XL, YL, XL2, YL2, YB;
	local BotReplicationInfo BRI;
	local String S, O, L;
	local Font CanvasFont;
	local bool bAdminPlayer;
	local PlayerPawn PlayerOwner;
	local int Time;
	local DeathMatchPlus DMP;
	local SPRI MPRI;
	local int FlagShift; // for country flag
	local color tempcolor;

	foreach PRI.ChildActors(Class'SPRI', MPRI)
	{
		//log("FOUND MPRI");
		break;
	}

	PlayerOwner = PlayerPawn(Owner);

	bAdminPlayer = PRI.bAdmin;

	// Draw Name
	if (PRI.PlayerName == PlayerOwner.PlayerReplicationInfo.PlayerName)
		Canvas.DrawColor = GoldColor;

	if ( bAdminPlayer )
      Canvas.DrawColor = WhiteColor;

	Canvas.SetPos(XOffset, YOffset);   // changing position of nickname
	Canvas.DrawText(PRI.PlayerName, False);
	Canvas.StrLen(PRI.PlayerName, XL, YB);

	if ( Canvas.ClipX > 512 )
	{
		CanvasFont = Canvas.Font;
		Canvas.Font = Font'SmallFont';
		Canvas.DrawColor = WhiteColor;

		if (Level.NetMode != NM_Standalone)
		{
			if ( !bCompressed || (Canvas.ClipX > 640) )
			{
				// Draw Time
				Time = Max(1, (Level.TimeSeconds + PlayerOwner.PlayerReplicationInfo.StartTime - PRI.StartTime)/60);
				Canvas.StrLen(TimeString$":     ", XL, YL);
				Canvas.SetPos(XOffset - XL - 6, YOffset);
				Canvas.DrawText(TimeString$":"@Time, false);
			}

			// Draw Ping
			Canvas.StrLen(PingString$":     ", XL2, YL2);
			Canvas.SetPos(XOffset - XL2 - 6, YOffset + (YL+1));
			Canvas.DrawText(PingString$":"@PRI.Ping, false);
		}
		Canvas.Font = CanvasFont;
	}

	// Draw Score
	if (PRI.PlayerName == PlayerOwner.PlayerReplicationInfo.PlayerName)
		Canvas.DrawColor = GoldColor;
	else
		Canvas.DrawColor = TeamColor[PRI.Team];
	DrawScore(Canvas, PRI.Score, XOffset, YOffset);

	if (Canvas.ClipX < 512)
		return;

	// Draw the country flag
	if(MPRI != None)
	{
		if(MPRI.CountryPrefix != "")
		{
			Canvas.bNoSmooth = False;
			tempcolor=Canvas.DrawColor;
			Canvas.DrawColor = WhiteColor;
			FlagShift=19;
			Canvas.SetPos(XOffset + Canvas.ClipX / 4.5 - 26, YOffset -2 + YL); //changing postiton of country flag
			Canvas.DrawIcon(FD[GetFlagIndex(MPRI.CountryFlagsPackage, MPRI.CountryPrefix)].Tex, 1.0);
			Canvas.bNoSmooth = True;
			Canvas.DrawColor=tempcolor;
		}
	}

	// Draw location, Order
	if ( !bCompressed && (PRI.Team == OwnerInfo.Team) )
	{
		CanvasFont = Canvas.Font;
		Canvas.Font = Font'SmallFont';

		if ( PRI.PlayerLocation != None )
			L = PRI.PlayerLocation.LocationName;
		else if ( PRI.PlayerZone != None )
			L = PRI.PlayerZone.ZoneName;
		else
			L = "";
		if ( L != "" )
		{
			L = InString@L;
			Canvas.SetPos(XOffset, YOffset + YB);
			Canvas.DrawText(L, False);
		}
		if(PRI.bIsABot)
		{
			O = OwnerGame.GetOrderString(PRI);
			if (O != "")
			{
				O = OrdersString@O;
				Canvas.StrLen(O, XL2, YL2);
				Canvas.SetPos(XOffset, YOffset + YB + YL2);
				Canvas.DrawText(O, False);
			}
		}
		Canvas.Font = CanvasFont;
	}

	if ( (PRI.HasFlag != None) && PRI.HasFlag.IsA('PureFlag') )
	{
		Canvas.DrawColor=WhiteColor;
		Canvas.Style=1;
		Canvas.SetPos(XOffset - 32,YOffset);
		Canvas.DrawIcon(Texture'Botpack.Icons.GreenFlag',1.00);
	}

	if ( (Canvas.ClipX > 512) && Level.NetMode != NM_Standalone)
	{
		CanvasFont = Canvas.Font;
		Canvas.Font = Font'SmallFont';
		Canvas.DrawColor = WhiteColor;

		if ( !bCompressed || (Canvas.ClipX > 640) )
		{
			// Draw Packetloss
			Canvas.StrLen(LossString$":    %", XL, YL);
			Canvas.SetPos(XOffset - XL - 6, YOffset + (YL+1)*2);
			Canvas.DrawText(LossString$":"@PRI.PacketLoss$"%", false);
		}

		if ( !bCompressed )
		{
			if(MPRI != None) // set in top of this function
			{
				if (MPRI.ShowBonusInfo == 1)
				{
					Canvas.StrLen("Nspd:     ", XL, YL);
					Canvas.SetPos(XOffset - XL - 6, YOffset + (YL+1)*3);
					Canvas.DrawText("Nspd:"@MPRI.Info, false);
				}
				else if (MPRI.ShowBonusInfo == 2)
				{
					Canvas.StrLen("Eff.:    %", XL, YL);
					Canvas.SetPos(XOffset - XL - 6, YOffset + (YL+1)*3);
					Canvas.DrawText("Eff.:"@MPRI.Info$"%", false);
				}
			}
		}
		Canvas.Font = CanvasFont;
	}
}

function DrawTrailer( canvas Canvas )
{
	local int Hours, Minutes, Seconds;
	local float XL, YL;
	local float DummyX, DummyY, Nil, X1, Y1;
	local PlayerPawn PlayerOwner;
	local string TimeStr, HeaderText;
	local PlayerReplicationInfo PRI;

	Canvas.bCenter = true;
	Canvas.StrLen("Test", XL, YL);
	Canvas.DrawColor = GreenColor;
	PlayerOwner = PlayerPawn(Owner);
	Canvas.SetPos(0, Canvas.ClipY - YL);
	if ( (Level.NetMode == NM_Standalone) && Level.Game.IsA('DeathMatchPlus') )
	{
		if ( DeathMatchPlus(Level.Game).bRatedGame )
			Canvas.DrawText(DeathMatchPlus(Level.Game).RatedGameLadderObj.SkillText@PlayerOwner.GameReplicationInfo.GameName@MapTitle@MapTitleQuote$Level.Title$MapTitleQuote, true);
		else if ( DeathMatchPlus(Level.Game).bNoviceMode ) 
			Canvas.DrawText(class'ChallengeBotInfo'.default.Skills[Level.Game.Difficulty]@PlayerOwner.GameReplicationInfo.GameName@MapTitle@MapTitleQuote$Level.Title$MapTitleQuote, true);
		else  
			Canvas.DrawText(class'ChallengeBotInfo'.default.Skills[Level.Game.Difficulty + 4]@PlayerOwner.GameReplicationInfo.GameName@MapTitle@MapTitleQuote$Level.Title$MapTitleQuote, true);
	}
	else
		Canvas.DrawText(Playing@PlayerOwner.GameReplicationInfo.GameName@MapTitle@Level.Title@ "on" @ (PlayerPawn(Owner).GameReplicationInfo.ServerName), true);

	Canvas.DrawColor = Pink;
	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.SetPos(0, Canvas.ClipY - 3 * YL);
	Canvas.DrawText("Map Author: "$Level.Author, true);



	TGRI = TournamentGameReplicationInfo(PlayerPawn(Owner).GameReplicationInfo);
	if ( TGRI == None )
		return;

  // Draw Time
  if( ( PlayerPawn(Owner).GameReplicationInfo.RemainingTime > 0 ) )
  {

	if( PlayerPawn(Owner).GameReplicationInfo.RemainingTime <= 0 )
	{
	  TimeStr = RemainingTime $ "00:00";
	}
	else
	{
	  Minutes = PlayerPawn(Owner).GameReplicationInfo.RemainingTime / 60;
	  Seconds = PlayerPawn(Owner).GameReplicationInfo.RemainingTime % 60;
	  TimeStr = RemainingTime $ TwoDigitString( Minutes ) $ ":" $ TwoDigitString( Seconds );
	}
  }
  else
  {
	Seconds = PlayerPawn(Owner).GameReplicationInfo.ElapsedTime;
	Minutes = Seconds / 60;
	Hours = Minutes / 60;
	Seconds = Seconds - ( Minutes * 60 );
	Minutes = Minutes - ( Hours * 60 );
	TimeStr = ElapsedTime $ TwoDigitString( Hours ) $ ":" $ TwoDigitString( Minutes ) $ ":" $ TwoDigitString( Seconds );
  }
	ForEach AllActors (class'PlayerReplicationInfo', PRI)
	{
		if ( PRI.bIsSpectator && !PRI.bWaitingPlayer && PRI.StartTime > 0)
		{
			if(HeaderText=="") HeaderText = pri.Playername;
			else HeaderText = HeaderText$", "$pri.Playername;
		}
	}
	if (HeaderText=="") HeaderText = "No one spectating this match."; else HeaderText = HeaderText$".";																													 //  but I'm to lazy rewrite it :P
																														 // Atleast it's working..
		Canvas.SetPos(0, Canvas.ClipY - 2 * YL);
		Canvas.DrawColor = Orange;
		Canvas.DrawText( "Current Time:" @ GetTimeStr() @ "|" @ FragGoal@TGRI.GoalTeamScore @ "|" @ "Time Limit:" @TGRI.TimeLimit$":00" @ "|" @ TimeStr );

  // Draw Spectators
		Canvas.StrLen( HeaderText, DummyX, Nil );
		Canvas.Style = ERenderStyle.STY_Normal;
		Canvas.SetPos(0, Canvas.ClipY - 4.2 * YL);
  		Canvas.DrawColor = GoldColor;
		Canvas.DrawText("Spectators:"@HeaderText);

  HeaderText="";

		Canvas.bCenter = False;
}

function string GetTimeStr()
{
  local string Mon, Day, Hour, Min;

  Hour = string( PlayerPawn( Owner ).Level.Hour );
  if( int( Hour ) < 10 ) Hour = "0" $ Hour;
  
  Min = string( PlayerPawn( Owner ).Level.Minute );
  if( int( Min ) < 10 ) Min = "0" $ Min;

  switch( PlayerPawn( Owner ).Level.month )
  {
	case  1: Mon = "Jan"; break;
	case  2: Mon = "Feb"; break;
	case  3: Mon = "Mar"; break;
	case  4: Mon = "Apr"; break;
	case  5: Mon = "May"; break;
	case  6: Mon = "Jun"; break;
	case  7: Mon = "Jul"; break;
	case  8: Mon = "Aug"; break;
	case  9: Mon = "Sep"; break;
	case 10: Mon = "Oct"; break;
	case 11: Mon = "Nov"; break;
	case 12: Mon = "Dec"; break;
  }

  switch( PlayerPawn( Owner ).Level.dayOfWeek )
  {
	case 0: Day = "Sunday";	break;
	case 1: Day = "Monday";	break;
	case 2: Day = "Tuesday";   break;
	case 3: Day = "Wednesday"; break;
	case 4: Day = "Thursday";  break;
	case 5: Day = "Friday";	break;
	case 6: Day = "Saturday";  break;
  }

  return Day @ PlayerPawn( Owner ).Level.Day @ Mon @ PlayerPawn( Owner ).Level.Year $ "," @ Hour $ ":" $Min;
}

defaultproperties
{
	FragGoal="Team Score Limit:"
     White=(R=255,G=255,B=255)
     Pink=(R=242,G=128,B=249)
     TealColor=(G=255,B=255)
     Orange=(R=255,G=128)
     PurpleColor=(R=180,G=64,B=255)
     RedColor=(R=255)
     BlueColor=(B=255)
     GreenColor=(G=255)
     GoldColor=(R=255,G=255)
     Texture=None
}
