class Nali_Rabbit extends ScriptedPawn;

var() byte BiteDamage;
var() byte LungeDamage;
var(Sounds) sound Call;
var(Sounds) sound Bite;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	MaxDesiredSpeed = 0.7 + 0.1 * skill;
}

function JumpOffPawn()
{
	Super.JumpOffPawn();
	PlayAnim('Jump', 1.0, 0.2);
}

function SetMovementPhysics()
{
	SetPhysics(PHYS_Falling); 
}

function PlayWaiting()
	{
	local float decision;
	local float animspeed;
	animspeed = 0.4 + 0.6 * FRand(); 
	decision = FRand();
	if ( !bool(NextAnim) || (decision < 0.4) )
	{
		if ( !bQuiet )
			PlaySound(Call, SLOT_Interact);
		NextAnim = 'Eat';
	}
	else if (decision < 0.55)
		NextAnim = 'Looking';
	else if (decision < 0.7)
	{
		if ( !bQuiet )
			PlaySound(Call, SLOT_Interact);
		NextAnim = 'Eat';
	}
	else if (decision < 0.7)
		NextAnim = 'Call';
	else 
		NextAnim = 'Looking';
		
	LoopAnim(NextAnim, animspeed);
	}

function PlayPatrolStop()
{
	PlayWaiting();
}

function PlayWaitingAmbush()
{
	PlayWaiting();
}

function PlayChallenge()
{
	if ( FRand() < 0.3 )
		PlayWaiting();
	else
		PlayAnim('Looking', 0.2 + 0.3 * FRand());
}

function TweenToFighter(float tweentime)
{
	TweenAnim('Looking', tweentime);
}

function TweenToRunning(float tweentime)
{
	if (AnimSequence != 'Jump' || !bAnimLoop)
	TweenAnim('Jump', tweentime);
}

function TweenToWalking(float tweentime)
{
	TweenAnim('Jump', tweentime);
}

function TweenToWaiting(float tweentime)
{
	TweenAnim('Looking', tweentime);
}

function TweenToPatrolStop(float tweentime)
{
	TweenAnim('Looking', tweentime);
}

function PlayRunning()
{
	LoopAnim('Jump', 1.5);
}

function PlayWalking()
{
	LoopAnim('Jump');
}

function PlayThreatening()
{
	PlayWaiting();
}

function PlayTurning()
{
	TweenAnim('Jump', 0.3);
}

function PlayDying(name DamageType, vector HitLocation)
{
	local CreatureChunks carc;
	local Pawn aPawn;

		Spawn(class'BloodBurst');
		carc = Spawn(class 'CreatureChunks',,, Location);
		if (carc != None)
		{
			carc.TrailSize = 3;
			carc.Mesh = mesh'CowBody1';
			carc.DrawScale = 0.65;
			carc.Initfor(self);
		}
		PlaySound(sound'gibP6',, 3.0);
		Destroy();
}

function PlayTakeHit(float tweentime, vector HitLoc, int damage)
{
	PlayAnim('Land');
}

function PlayVictoryDance()
{
	PlayAnim('Looking', 1.0, 0.1);
}

function PlayMeleeAttack()
{
	local float dist, decision;

	decision = FRand();
	dist = VSize(Target.Location - Location);
	if (dist > CollisionRadius + Target.CollisionRadius + 45)
		decision = 0.0;

	if (Physics == PHYS_Falling)
		decision = 1.0;
	if (Target == None)
		decision = 1.0;

	if (decision < 0.15)
	{
		Enable('Bump');
		PlayAnim('JumpUp');
		Velocity = 450 * Normal(Target.Location + Target.CollisionHeight * vect(0,0,0.75) - Location);
		if (dist > CollisionRadius + Target.CollisionRadius + 35)
			Velocity.Z += 0.7 * dist;
		SetPhysics(PHYS_Falling);
	}
  	else
  	{
		PlaySound(Bite, SLOT_Interact);
  		PlayAnim('Call');
		MeleeRange = 50;
		MeleeDamageTarget(BiteDamage, vect(0,0,0));
		MeleeRange = Default.MeleeRange;
	}  		
}

state MeleeAttack
{
ignores SeePlayer, HearNoise;

	singular function Bump(actor Other)
	{
		Disable('Bump');
		if ( (Other == Target) && (AnimSequence == 'JumpUp') )
			if (MeleeDamageTarget(LungeDamage, vect(0,0,0)))
			{
				if (FRand() < 0.5)
					PlaySound(Bite, SLOT_Interact);
				else
					PlaySound(Bite, SLOT_Interact);
			}
	}
}		

auto state StartUp
{
	function SetMovementPhysics()
	{
		SetPhysics(PHYS_None);
	}
}


state Waiting
{
TurnFromWall:
	if ( NearWall(70) )
	{
		PlayTurning();
		TurnTo(Focus);
	}
Begin:
	TweenToWaiting(0.4);
	bReadyToAttack = false;
	if (Physics != PHYS_Falling) 
		SetPhysics(PHYS_None);
KeepWaiting:
	NextAnim = '';
}

defaultproperties
{
     BiteDamage=10
     LungeDamage=10
     Call=Sound'CallBn'
     Bite=Sound'munch1p'
     Aggressiveness=10.000000
     bCanStrafe=True
     MeleeRange=280.000000
     GroundSpeed=260.000000
     WaterSpeed=100.000000
     JumpZ=340.000000
     Visibility=100
     SightRadius=3000.000000
     PeripheralVision=-0.400000
     Health=20
     Intelligence=BRAINS_NONE
     CombatStyle=1.000000
     DrawType=DT_Mesh
     Mesh=LodMesh'UnrealShare.Rabbit'
     CollisionRadius=18.299999
     CollisionHeight=13.300000
     Mass=80.000000
     RotationRate=(Pitch=3072,Yaw=65000,Roll=0)
}
